@@ -20,7 +20,7 @@
  * USA
  *
  * --------------------------------------------------------------------------
- * $Id: JadortServiceStatefulBean.java 4310 2009-07-07 14:41:06Z chahinem $
+ * $Id: JadortServiceStatefulBean.java 5655 2009-11-30 15:47:24Z alitokmen $
  * --------------------------------------------------------------------------
  */
 package org.ow2.jasmine.jadort.service.implementation;
@@ -40,8 +40,6 @@ import java.util.Set;
 
 import javax.annotation.PostConstruct;
 import javax.annotation.PreDestroy;
-import javax.ejb.Local;
-import javax.ejb.Remote;
 import javax.ejb.Stateful;
 import javax.xml.bind.UnmarshalException;
 
@@ -61,6 +59,7 @@ import org.ow2.jasmine.jadort.api.entities.topology.ServerBean;
 import org.ow2.jasmine.jadort.api.entities.topology.TopologyBean;
 import org.ow2.jasmine.jadort.api.entities.topology.VMBean;
 import org.ow2.jasmine.jadort.api.entities.topology.WorkerBean;
+import org.ow2.jasmine.jadort.api.topology.TopologyReader;
 import org.ow2.jasmine.jadort.service.action.ServerAction;
 import org.ow2.jasmine.jadort.service.action.VMMAction;
 import org.ow2.jasmine.jadort.service.action.WorkerAction;
@@ -72,17 +71,12 @@ import org.ow2.jasmine.jadort.service.action.WorkerAction;
  * @author Remy Bresson
  * @author S. Ali Tokmen
  */
-@Local(value = IJadortService.class)
-@Remote(value = IJadortService.class)
 @Stateful(mappedName = IJadortService.EJB_JNDI_NAME)
 public class JadortServiceStatefulBean extends StepManager implements IJadortService {
 
     // Constants used internally by JadortServiceStatefulBean
     private final static int BUFFER_SIZE = 16384;
 
-    // Constants used for tests
-    static boolean DUMMY = false;
-
     @PostConstruct
     protected void initialize() {
         try {
@@ -132,7 +126,7 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException if the getOperationsList() method can not
      *         be called in the current step. (this method can be called only in
-     *         SELECT_OPERATION step)
+     *         the {@link Step#SELECT_OPERATION} step)
      */
     @SuppressWarnings("unchecked")
     public List<OperationStateBean> getOperationsList() throws JadortServiceException {
@@ -164,7 +158,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the selectOperation can not be called in the current step.
-     *         (this method can be called only in SELECT_OPERATION step)
+     *         (this method can be called only in the
+     *         {@link Step#SELECT_OPERATION} step)
      *         <li>if the operation to delete is still in progress in a critical
      *         phase and therefore cannot be deleted
      *         </ul>
@@ -194,8 +189,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException if the selectOperation can not be called
-     *         in the current step. (this method can be called only in
-     *         SELECT_OPERATION step)
+     *         in the current step. (this method can be called only in the
+     *         {@link Step#SELECT_OPERATION} step)
      */
     public void selectOperation(final OperationStateBean selectedOperation) throws JadortServiceException {
         synchronized (this) {
@@ -214,7 +209,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the selectOperation can not be called in the current step.
-     *         (this method can be called only in SELECT_OPERATION step)
+     *         (this method can be called only in the
+     *         {@link Step#SELECT_OPERATION} step)
      *         </ul>
      */
     public void createNewOperation(final String newOperationName) throws JadortServiceException {
@@ -353,8 +349,9 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException <ul>
-     *         <li>if the loadTopology method is called outside the
-     *         INITIALIZE_TOPOLOGY step,
+     *         <li>if the loadTopology can not be called in the current step.
+     *         (this method can be called only in the
+     *         {@link Step#INITIALIZE_TOPOLOGY} step)
      *         <li>if an error is occurred when parsing the .xml file
      *         </ul>
      */
@@ -363,20 +360,79 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
             this.refreshOperation();
             this.checkStep(Step.INITIALIZE_TOPOLOGY);
 
-            TopologyBean topology;
+            TopologyBean topology = null;
+            Exception jadortFormatException = null;
+            Exception deployMEFormatException = null;
             try {
                 topology = TopologyReader.loadJadortSpecificTopology(xmlTopoFile);
-            } catch (UnmarshalException e1) {
+            } catch (Exception e1) {
+                jadortFormatException = e1;
                 try {
                     topology = TopologyReader.loadDeployMETopology(xmlTopoFile);
-                } catch (UnmarshalException e2) {
-                    throw new JadortServiceException(xmlTopoFile.getName()
-                        + " is not a valid topology file.\n\nHere are some potentially helpful error messages:\n\t["
-                        + TopologyReader.XSD_PATH_JADORT + "] " + e1.getMessage() + "\n\t[" + TopologyReader.XSD_PATH_DEPLOYME
-                        + "] " + e2.getMessage(), null);
+                } catch (Exception e2) {
+                    deployMEFormatException = e2;
                 }
             }
 
+            if (topology == null) {
+                StringBuilder sb = new StringBuilder();
+
+                if (jadortFormatException != null && jadortFormatException instanceof UnmarshalException
+                    && deployMEFormatException != null && deployMEFormatException instanceof UnmarshalException) {
+                    // XML doesn't match any XSD
+                    UnmarshalException e1 = (UnmarshalException) jadortFormatException;
+                    if (e1.getLinkedException() != null) {
+                        e1 = new UnmarshalException(e1.getLinkedException().getMessage(), e1);
+                    }
+                    UnmarshalException e2 = (UnmarshalException) deployMEFormatException;
+                    if (e2.getLinkedException() != null) {
+                        e2 = new UnmarshalException(e2.getLinkedException().getMessage(), e2);
+                    }
+
+                    sb.append(xmlTopoFile.getName());
+                    sb.append(" is not a valid JASMINe Deploy ME or JaDOrT topology file.");
+                    sb.append("\n\nHere are some potentially helpful error messages:");
+                    sb.append("\n\t(For the JaDOrT-specific format: ");
+                    sb.append(e1.getMessage());
+                    sb.append("\n\t(For the JASMINe DeployME format: ");
+                    sb.append(e2.getMessage());
+
+                    throw new JadortServiceException(sb.toString(), null);
+                }
+
+                if (deployMEFormatException != null && jadortFormatException != null
+                    && jadortFormatException instanceof UnmarshalException) {
+                    // XML matches Deploy ME XSD but has another problem
+                    sb.append(xmlTopoFile.getName());
+                    sb.append(" is not a valid JASMINe Deploy ME topology file.\n\n");
+                    sb.append(deployMEFormatException.getMessage());
+
+                    throw new JadortServiceException(sb.toString(), deployMEFormatException);
+                }
+
+                if (jadortFormatException != null && deployMEFormatException != null
+                    && deployMEFormatException instanceof UnmarshalException) {
+                    // XML matches JaDOrT XSD but has another problem
+                    sb.append(xmlTopoFile.getName());
+                    sb.append(" is not a valid JaDOrT topology file.\n\n");
+                    sb.append(jadortFormatException.getMessage());
+
+                    throw new JadortServiceException(sb.toString(), jadortFormatException);
+                }
+
+                sb.append("Cannot load topology file ");
+                sb.append(xmlTopoFile);
+
+                if (jadortFormatException != null) {
+                    throw new JadortServiceException(sb.toString(), jadortFormatException);
+                }
+                if (deployMEFormatException != null) {
+                    throw new JadortServiceException(sb.toString(), deployMEFormatException);
+                }
+
+                throw new JadortServiceException(sb.toString(), null);
+            }
+
             this.operation.setTopology(topology);
             this.mergeOperation();
         }
@@ -387,7 +443,7 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException if the selectedGroup method can not be
      *         called in the current step. (this method can be called only in
-     *         SELECT_GROUP step)
+     *         the {@link Step#SELECT_GROUP} step)
      */
     public void selectGroup(final GroupBean selectedGroup) throws JadortServiceException {
         synchronized (this) {
@@ -411,7 +467,7 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException if the selectedGroup method can not be
      *         called in the current step. (this method can be called only in
-     *         SELECT_OPERATION_TYPE step)
+     *         the {@link Step#SELECT_OPERATION_TYPE} step)
      */
     public void selectOperationType(final OperationType operationType) throws JadortServiceException {
         synchronized (this) {
@@ -492,7 +548,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the selectApplication can not be called in the current
-     *         step. (this method can be called only in SELECT_APPLICATION step)
+     *         step. (this method can be called only in the
+     *         {@link Step#SELECT_APPLICATION} step)
      *         </ul>
      */
     public void selectApplication(final URL url) throws JadortServiceException {
@@ -553,8 +610,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException<ul>
      *         <li>if the selectVMImage can not be called in the current step.
-     *         (this method can be called only in SELECT_VM_IMAGE_FOR_SERVER
-     *         step)
+     *         (this method can be called only in the
+     *         {@link Step#SELECT_VM_IMAGE_FOR_SERVER} step)
      *         <li>or if selectedVMImage is not known
      *         <li>or if fetching fails
      *         </ul>
@@ -590,8 +647,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException<ul>
      *         <li>if the selectVMImage can not be called in the current step.
-     *         (this method can be called only in SELECT_VM_IMAGE_FOR_SERVER
-     *         step)
+     *         (this method can be called only in the
+     *         {@link Step#SELECT_VM_IMAGE} step)
      *         <li>or if selectedVMImage is not known
      *         <li>or if server is not known
      *         <li>or if fetching fails
@@ -628,8 +685,8 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException if the selectServers method can not be
-     *         called in the current step (this method can be called only in
-     *         SELECT_SERVERS step)
+     *         called in the current step (this method can be called only in the
+     *         {@link Step#SELECT_SERVERS} step)
      */
     public void selectServers(final List<ServerBean> selectedServers) throws JadortServiceException {
         synchronized (this) {
@@ -723,16 +780,18 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException If the abortServer can not be called in
-     *         the current step. (this method can be called only in
-     *         Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION,
-     *         EXECUTING_MAINTENANCE_CLUSTER,
-     *         Step.EXECUTING_MAINTENANCE_NO_CLUSTER steps)
+     *         the current step. (this method can be called only in the
+     *         {@link Step#EXECUTING_MIGRATION},
+     *         {@link Step#UNDEPLOY_ERASE_OLD_VERSION},
+     *         {@link Step#EXECUTING_MIGRATION_OSGI},
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      */
     public void abortServer(final ServerBean server) throws JadortServiceException {
         synchronized (this) {
             this.refreshOperation();
-            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MAINTENANCE_CLUSTER,
-                Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
+            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MIGRATION_OSGI,
+                Step.EXECUTING_MAINTENANCE_CLUSTER, Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
 
             ServerProgressBean serverProgress = this.getServerProgress(server);
             if (!serverProgress.getActionState().equals(ActionState.RUNNING)) {
@@ -753,17 +812,20 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the restartServermethod can not be called in the current
-     *         step. (this method can be called only in EXECUTING_MIGRATION,
-     *         UNDEPLOY_ERASE_OLD_VERSION, EXECUTING_MAINTENANCE_CLUSTER,
-     *         EXECUTING_MAINTENANCE_NO_CLUSTER and DESTROY_OLD_VM_HOSTS steps)
+     *         step. (this method can be called only in the
+     *         {@link Step#EXECUTING_MIGRATION},
+     *         {@link Step#UNDEPLOY_ERASE_OLD_VERSION},
+     *         {@link Step#EXECUTING_MIGRATION_OSGI},
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      *         <li>if the server state is not in error state.
      *         </ul>
      */
     public void restartServer(final ServerBean server) throws JadortServiceException {
         synchronized (this) {
             this.refreshOperation();
-            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MAINTENANCE_CLUSTER,
-                Step.EXECUTING_MAINTENANCE_NO_CLUSTER, Step.DESTROY_OLD_VM_HOSTS);
+            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MIGRATION_OSGI,
+                Step.EXECUTING_MAINTENANCE_CLUSTER, Step.EXECUTING_MAINTENANCE_NO_CLUSTER, Step.DESTROY_OLD_VM_HOSTS);
 
             ServerProgressBean serverProgress = this.getServerProgress(server);
             if (!serverProgress.getActionState().equals(ActionState.FINISHED_ERROR)) {
@@ -783,16 +845,18 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException if the checkServer can not be called in
-     *         the current step. (this method can be called only in
-     *         EXECUTING_MIGRATION, UNDEPLOY_ERASE_OLD_VERSION,
-     *         EXECUTING_MAINTENANCE_CLUSTER and
-     *         EXECUTING_MAINTENANCE_NO_CLUSTER steps)
+     *         the current step. (this method can be called only in the
+     *         {@link Step#EXECUTING_MIGRATION},
+     *         {@link Step#UNDEPLOY_ERASE_OLD_VERSION},
+     *         {@link Step#EXECUTING_MIGRATION_OSGI},
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      */
     public boolean checkServer(final ServerBean server) throws JadortServiceException {
         synchronized (this) {
             this.refreshOperation();
-            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MAINTENANCE_CLUSTER,
-                Step.EXECUTING_MAINTENANCE_NO_CLUSTER, Step.DESTROY_OLD_VM_HOSTS);
+            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MIGRATION_OSGI,
+                Step.EXECUTING_MAINTENANCE_CLUSTER, Step.EXECUTING_MAINTENANCE_NO_CLUSTER, Step.DESTROY_OLD_VM_HOSTS);
 
             try {
                 ServerProgressBean serverProgress = this.getServerProgress(server);
@@ -968,17 +1032,20 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the ignoreServer can not be called in the current step.
-     *         (this method can be called only in Step.EXECUTING_MIGRATION,
-     *         UNDEPLOY_ERASE_OLD_VERSION EXECUTING_MAINTENANCE_CLUSTER and
-     *         EXECUTING_MAINTENANCE_NO_CLUSTER steps)
+     *         (this method can be called only in the
+     *         {@link Step#EXECUTING_MIGRATION},
+     *         {@link Step#UNDEPLOY_ERASE_OLD_VERSION},
+     *         {@link Step#EXECUTING_MIGRATION_OSGI},
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      *         <li>if the server state is not in error state.
      *         </ul>
      */
     public void ignoreServer(final ServerBean server) throws JadortServiceException {
         synchronized (this) {
             this.refreshOperation();
-            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MAINTENANCE_CLUSTER,
-                Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
+            this.checkStep(Step.EXECUTING_MIGRATION, Step.UNDEPLOY_ERASE_OLD_VERSION, Step.EXECUTING_MIGRATION_OSGI,
+                Step.EXECUTING_MAINTENANCE_CLUSTER, Step.EXECUTING_MAINTENANCE_NO_CLUSTER);
 
             ServerProgressBean serverProgress = this.getServerProgress(server);
 
@@ -1013,9 +1080,9 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException If the abortWorker can not be called in
-     *         the current step. (this method can be called only in
-     *         EXECUTING_MAINTENANCE_CLUSTER and
-     *         EXECUTING_MAINTENANCE_NO_CLUSTER step)
+     *         the current step. (this method can be called only in the
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      */
     public void abortWorker(final WorkerBean worker) throws JadortServiceException {
         synchronized (this) {
@@ -1041,9 +1108,9 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the restartWorkermethod can not be called in the current
-     *         step. (this method can be called only in
-     *         EXECUTING_MAINTENANCE_CLUSTER and
-     *         EXECUTING_MAINTENANCE_NO_CLUSTER steps)
+     *         step. (this method can be called only in the
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      *         <li>if the worker state is not in error state.
      *         </ul>
      */
@@ -1070,9 +1137,9 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * {@inheritDoc}
      * 
      * @throws JadortServiceException if the checkWorker can not be called in
-     *         the current step. (this method can be called only in
-     *         EXECUTING_MAINTENANCE_CLUSTER and
-     *         EXECUTING_MAINTENANCE_NO_CLUSTER steps)
+     *         the current step. (this method can be called only in the
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      */
     public boolean checkWorker(final WorkerBean worker) throws JadortServiceException {
         synchronized (this) {
@@ -1104,8 +1171,9 @@ public class JadortServiceStatefulBean extends StepManager implements IJadortSer
      * 
      * @throws JadortServiceException <ul>
      *         <li>if the ignoreWorker can not be called in the current step.
-     *         (this method can be called only in EXECUTING_MAINTENANCE_CLUSTER
-     *         and EXECUTING_MAINTENANCE_NO_CLUSTER steps)
+     *         (this method can be called only in the
+     *         {@link Step#EXECUTING_MAINTENANCE_CLUSTER} and
+     *         {@link Step#EXECUTING_MAINTENANCE_NO_CLUSTER} steps)
      *         <li>if the worker state is not in error state.
      *         </ul>
      */
