@@ -20,7 +20,7 @@
  * USA
  *
  * --------------------------------------------------------------------------
- * $Id: JadortServiceTest.java 5638 2009-11-24 17:24:39Z alitokmen $
+ * $Id: JadortServiceTest.java 5656 2009-11-30 17:12:08Z alitokmen $
  * --------------------------------------------------------------------------
  */
 package org.ow2.jasmine.jadort.tests.implementation;
@@ -56,12 +56,12 @@ import org.ow2.jasmine.jadort.api.entities.topology.GroupBean;
 import org.ow2.jasmine.jadort.api.entities.topology.ServerBean;
 import org.ow2.jasmine.jadort.service.action.DummyServerAction;
 import org.ow2.jasmine.jadort.service.action.DummyWorkerAction;
-import org.ow2.jasmine.jadort.service.implementation.JadortServiceStatefulBean;
 import org.ow2.jasmine.jadort.service.implementation.JMSSender;
+import org.ow2.jasmine.jadort.service.implementation.JadortServiceStatefulBean;
 
 /**
  * Tests the JaDOrT service implementation.
- *
+ * 
  * @author Malek Chahine
  * @author Remy Bresson
  * @author S. Ali Tokmen
@@ -839,6 +839,56 @@ public class JadortServiceTest {
         }
     }
 
+    /**
+     * Integration test for application migration with an OSGi bundle.
+     */
+    @Test(timeout = JadortServiceTest.TEST_TIMEOUT * 1000)
+    public void testDeployOsgiBundle() throws Exception {
+        System.out.println("JadortServiceTest.testDeployOsgiBundle");
+
+        this.deployOsgiBundle("src/test/resources/service-alerte-1.0.0-SNAPSHOT/service-alerte.jar");
+        Assert.assertNull(this.jadortService.getCurrentOperation().getServerProgressList().iterator().next()
+            .getOldDeploymentItem());
+    }
+
+    /**
+     * Integration test for application migration with an OSGi bundle.
+     */
+    @Test(timeout = JadortServiceTest.TEST_TIMEOUT * 1000)
+    public void testDeployNewVersionOfOsgiBundle() throws Exception {
+        System.out.println("JadortServiceTest.testDeployNewVersionOfOsgiBundle");
+
+        this.deployOsgiBundle("src/test/resources/service-alerte-1.0.0/service-alerte.jar");
+        Assert.assertNotNull(this.jadortService.getCurrentOperation().getServerProgressList().iterator().next()
+            .getOldDeploymentItem());
+    }
+
+    private void deployOsgiBundle(final String bundlePath) throws Exception {
+        System.out.println("\tJadortServiceTest.deployOsgiBundle");
+
+        this.testSelectGroupMigrate();
+
+        this.jadortService.selectApplication(new File(bundlePath).toURI().toURL());
+
+        Assert.assertEquals(Step.SELECT_APPLICATION, this.jadortService.getCurrentOperation().getCurrentStep());
+        this.jadortService.next();
+
+        Assert.assertEquals(Step.SELECT_SERVERS, this.jadortService.getCurrentOperation().getCurrentStep());
+        List<ServerBean> serversToSelect = new ArrayList<ServerBean>();
+        serversToSelect.addAll(this.jadortService.getCurrentOperation().getSelectedGroup().getServers());
+        this.jadortService.selectServers(serversToSelect);
+        this.jadortService.next();
+
+        Assert.assertEquals(Step.EXECUTING_MIGRATION_OSGI, this.jadortService.getCurrentOperation().getCurrentStep());
+        this.waitForFinishAndCheckServerProgressList(ServerProgressState.INITIAL, 0);
+
+        this.executeOsgiMigration();
+
+        // Change page
+        this.jadortService.next();
+        Assert.assertEquals(Step.FINISHED, this.jadortService.getCurrentOperation().getCurrentStep());
+    }
+
     private void executeMigration() throws Exception {
         System.out.println("\tJadortServiceTest.executeMigration");
 
@@ -859,6 +909,32 @@ public class JadortServiceTest {
         Assert.assertEquals(Step.EXECUTING_MIGRATION, this.jadortService.getCurrentOperation().getCurrentStep());
     }
 
+    private void executeOsgiMigration() throws Exception {
+        System.out.println("\tJadortServiceTest.executeOsgiMigration");
+
+        // Upload bundle
+        this.jadortService.next();
+        Assert.assertEquals(Step.EXECUTING_MIGRATION_OSGI, this.jadortService.getCurrentOperation().getCurrentStep());
+        this.waitForFinishAndCheckServerProgressList(ServerProgressState.UPLOAD_OK, 25);
+
+        // Undeploy old version
+        this.jadortService.next();
+        Assert.assertEquals(Step.EXECUTING_MIGRATION_OSGI, this.jadortService.getCurrentOperation().getCurrentStep());
+        this.waitForFinishAndCheckServerProgressList(ServerProgressState.UNDEPLOY_OK, 50);
+
+        // Deploy bundle
+        this.jadortService.next();
+        Assert.assertEquals(Step.EXECUTING_MIGRATION_OSGI, this.jadortService.getCurrentOperation().getCurrentStep());
+        this.waitForFinishAndCheckServerProgressList(ServerProgressState.DEPLOY_OK, 75);
+
+        // Erase old version
+        this.jadortService.next();
+        Assert.assertEquals(Step.EXECUTING_MIGRATION_OSGI, this.jadortService.getCurrentOperation().getCurrentStep());
+        this.waitForFinishAndCheckServerProgressList(ServerProgressState.ERASE_OK, 100);
+
+        Assert.assertEquals(Step.EXECUTING_MIGRATION_OSGI, this.jadortService.getCurrentOperation().getCurrentStep());
+    }
+
     private void undeployEraseOldVersion(final boolean eraseOld) throws Exception {
         System.out.println("\tJadortServiceTest.undeployEraseOldVersion");
 
